import { EventEmitter } from './EventEmitter';
import { EventHistory, EventInfo, Hook, Hooks, Option } from './Interfaces';
import { AsyncListener, Listener } from './Types';
export declare class ListenerManager {
    private eventNamespaces;
    private readonly wildCardNamespace;
    private eventFilters;
    private executingListeners;
    private listenerQueue;
    private eventHistory;
    private hooks;
    /**
     * Retrieves the event history based on a specific event name.
     *
     * @param event - The name of the event to filter by. This can include a namespace (e.g., 'namespace.eventName').
     * @returns {Array<{ event: string, listenerId: string, timestamp: number, args: unknown[] }>}
     * - An array of objects that match the specified event name, each containing the event name, listener ID, timestamp, and arguments.
     */
    getSpecificEventHistory(event: string): Array<EventHistory>;
    /**
     * Retrieves all recorded event histories.
     *
     * @returns {Array<{ event: string, listenerId: string, timestamp: number, args: unknown[] }>}
     * - An array of objects, each containing the event name, listener ID, timestamp, and the arguments passed to the listener.
     */
    getAllEventHistory(): Array<EventHistory>;
    /**
     * @param emitter - The `EventEmitter` instance where the listener is being added.
     * @param event - The event name which can include a namespace (e.g., 'namespace.eventName').
     * @param listener - The listener function to be called when the event is emitted.
     *                  This can be either a `Listener` (synchronous function) or `AsyncListener` (asynchronous function).
     * @param option - An object containing various configuration options for the listener.
     * @param option.filter - Optional. A filter function to determine whether the listener should be executed.
     * @param option.throttle - Optional. Throttle configuration. Can be a number (delay in ms) or an object with { delay, leading, trailing }.
     *                          If specified, the listener will only be called at most once within the given time window.
     *                          - leading: If true, triggers on the leading edge. Default: true.
     *                          - trailing: If true, triggers on the trailing edge. Default: true.
     * @param option.debounce - Optional. Debounce configuration. Can be a number (delay in ms) or an object with { delay, leading, trailing }.
     *                          If specified, the listener will be delayed and only called after the delay has passed since the last event.
     *                          - leading: If true, triggers on the leading edge. Default: false.
     *                          - trailing: If true, triggers on the trailing edge. Default: true.
     * @param option.priority - Optional. Priority of the listener. Higher values execute first. Default is 0.
     * @param option.concurrency - Optional. Maximum number of listeners executed in parallel. Default is unlimited.
     * @param option.separator - Optional. Custom separator used for parsing the event. Default is the global separator.
     */
    addListener(emitter: EventEmitter, event: string, listener: Listener | AsyncListener, option: Option): void;
    /**
     * @param _emitter - The `EventEmitter` instance. This parameter is unused in this method, as we only need access to the event namespace.
     * @param event - The event name which can include a namespace (e.g., 'namespace.eventName').
     * @param listener - The listener function to be removed from the specified event.
     */
    removeListener(_emitter: EventEmitter, event: string, listener: Listener): void;
    /**
     * @param _emitter - The `EventEmitter` instance. This parameter is unused in this method, as we only need access to the event namespaces.
     * @param event - The event name which can include a namespace (e.g., 'namespace.eventName').
     * @param args - Additional arguments to be passed to the listeners when they are invoked.
     *
     * This method processes and executes listeners for specific namespaces and wildcard combinations.
     */
    executeListeners(_emitter: EventEmitter, event: string, args: unknown[]): Promise<void>;
    /**
     * Lists all event subscriptions, including event names and listener count.
     *
     * @returns {Array<{ event: string, listenerCount: number }>} - An array of objects, each containing event name and count of listeners.
     */
    listSubscriptions(): Array<{
        event: string;
        listenerCount: number;
    }>;
    /**
     * Inspects a specific event subscription, showing details of listeners.
     *
     * @param event - The event name, possibly with a namespace (e.g., 'namespace.eventName').
     * @returns {Array<Object>} - An array of objects, each containing listener details (id, priority, concurrency, eventInfo, listener).
     */
    inspectSubscription(event: string): Array<{
        id: string;
        eventInfo: EventInfo;
        listener: Listener;
        priority: number;
        concurrency: number;
    }>;
    /**
     * Remove a subscription from the ListenerManager.
     *
     * @param event - The event name which can include a namespace (e.g., 'namespace.eventName').
     * @param listenerId - The unique identifier of the listener to be removed.
     */
    removeSubscription(event: string, listenerId: string): void;
    /**
     * Records an event occurrence in the history log.
     *
     * @param event - The name of the event that was emitted, which can include a namespace (e.g., 'namespace.eventName').
     * @param listenerId - The unique identifier of the listener that was invoked.
     * @param args - The arguments that were passed to the listener when it was invoked.
     */
    private recordEventHistory;
    /**
     * Remove a subscription or listener from the ListenerManager.
     *
     * @param event - The event name which can include a namespace (e.g., 'namespace.eventName').
     * @param listenerOrId - The unique identifier of the listener to be removed, or the listener function itself.
     */
    private removeSubscriptionOrListener;
    /**
     * Executes specific listeners for the given namespace, event, and wildcard combination.
     * @param namespace - The namespace for which listeners should be executed.
     * @param checkEventName - The event name or wildcard to check for listeners.
     * @param eventName - The actual event name for emitting.
     * @param separator - Separator used for parsing the event (if applicable, default is '.').
     * @param args - Additional arguments to be passed to the listeners.
     * @returns A promise that resolves when all specific listeners have been executed.
     */
    private executeSpecificListeners;
    /**
     * Executes a listener with concurrency control and handles queuing if the concurrency limit is reached.
     *
     * @param listener - The listener function to execute, either throttled or async.
     * @param id - The unique identifier for the listener to track execution.
     * @param concurrency - The maximum number of concurrent executions allowed for this listener.
     * @param isThrottled - A flag indicating whether the listener is throttled.
     * @param eventName - The name of the event being emitted.
     * @param args - The arguments to pass to the listener function.
     */
    private processListener;
    /**
     * Dequeues and processes the next listener from the listener queue.
     *
     * @param id - The ID of the listener to be dequeued.
     * @returns {Promise<void>} A promise that resolves once the listener has been processed.
     */
    private dequeueNextListener;
    /**
     * Handles errors that occur during the execution of a listener.
     * @param eventName - The name of the event for which the listener encountered an error.
     * @param error - The error object representing the encountered error.
     */
    private handleListenerError;
    /**
     * Creates a throttled version of the given listener function, ensuring it is called at most once within a specified time interval.
     * Supports leading/trailing edge control.
     * @param fn - The listener function to be throttled.
     * @param delay - The time delay (in milliseconds) between allowed invocations of the throttled function.
     * @param leading - If true, triggers on the leading edge.
     * @param trailing - If true, triggers on the trailing edge.
     * @returns A throttled listener function.
     */
    private throttle;
    /**
     * Creates a debounced version of the given listener function, ensuring it is called after a specified delay with no new invocations.
     * Supports leading/trailing edge control.
     * @param fn - The listener function to be debounced.
     * @param delay - The time delay (in milliseconds) before the debounced function is called after the last invocation.
     * @param leading - If true, triggers on the leading edge.
     * @param trailing - If true, triggers on the trailing edge.
     * @returns A debounced listener function.
     */
    private debounce;
    /**
     * Adds a lifecycle hook to be executed before or after an event is emitted.
     * @param type - The type of hook, either 'beforeEmit' or 'afterEmit'.
     * @param hook - The function to be executed.
     */
    addHook(type: keyof Hooks, hook: Hook): void;
    /**
     * Removes a previously added lifecycle hook.
     * @param type - The type of hook to remove.
     * @param hook - The hook function to remove.
     */
    removeHook(type: keyof Hooks, hook: Hook): void;
}
